module HashingTrickMl
  module ActsAsVectorized
    extend ActiveSupport::Concern

    included do
    end

    module ClassMethods
      def acts_as_vectorized(options = {})
        cattr_accessor :default_dimensions
        self.default_dimensions = options[:default_dimensions] || 1_000
      end
    end

    def build_word_vector(data, dimensions: self.class.default_dimensions, separator: ' ')
      words = normalize_words(data).split(separator)
      words.each_with_object([0] * dimensions) do |word, result|
        result[Digest::SHA2.digest(word.downcase).last(4).unpack('N1').first % dimensions] += 1
      end
    end

    def build_boolean_vector(subset, full_set)
      subset.each_with_object([0] * full_set.size) do |value, result|
        index = full_set.index(value)
        result[index] = 1 if index.present?
      end
    end

    def build_fuzzy_vector(statement)
      statement ? 1 : 0
    end

    def build_exponential_vector(values, dimensions:)
      values.reject(&:blank?).each_with_object([0] * dimensions) do |value, result|
        index = Math.log(value.to_f, 1.5)
        index = 0 if index.negative?
        index = dimensions - 1 if index > dimensions - 1

        result[index.floor] += 1
      end
    end

    def build_maybe_nil_vector(value)
      [value.presence || 0, value.present? ? 1 : 0]
    end

    private

    def normalize_words(data)
      strip_tags(data || '').gsub(/[^[:alpha:][:digit:]]+/, ' ').downcase
    end

    def strip_tags(html)
      ActionController::Base.helpers.strip_tags(html)
    end
  end
end
